import { Canvas } from "./Canvas";
import type { IRepulse } from "./Interfaces/IRepulse";
import type { IBubble } from "./Interfaces/IBubble";
import type { IContainerInteractivity } from "./Interfaces/IContainerInteractivity";
import { Particles } from "./Particles";
import { Retina } from "./Retina";
import type { IOptions } from "../Options/Interfaces/IOptions";
import { FrameManager } from "./FrameManager";
import type { RecursivePartial } from "../Types";
import { Options } from "../Options/Classes/Options";
import type { IContainerPlugin } from "./Interfaces/IContainerPlugin";
import type { IShapeDrawer } from "./Interfaces/IShapeDrawer";
import { Particle } from "./Particle";
import type { INoiseValue } from "./Interfaces/INoiseValue";
import type { INoise } from "./Interfaces/INoise";
import type { IAttract } from "./Interfaces/IAttract";
export declare class Container {
    readonly id: string;
    readonly sourceOptions?: RecursivePartial<IOptions> | undefined;
    started: boolean;
    destroyed: boolean;
    density: number;
    pageHidden: boolean;
    lastFrameTime: number;
    fpsLimit: number;
    interactivity: IContainerInteractivity;
    bubble: IBubble;
    repulse: IRepulse;
    attract: IAttract;
    readonly options: Options;
    actualOptions: Options;
    readonly retina: Retina;
    readonly canvas: Canvas;
    readonly particles: Particles;
    readonly drawer: FrameManager;
    readonly drawers: Map<string, IShapeDrawer>;
    readonly plugins: Map<string, IContainerPlugin>;
    readonly noise: INoise;
    private paused;
    private firstStart;
    private drawAnimationFrame?;
    private readonly eventListeners;
    private readonly intersectionObserver?;
    constructor(id: string, sourceOptions?: RecursivePartial<IOptions> | undefined, ...presets: string[]);
    play(force?: boolean): void;
    pause(): void;
    draw(): void;
    getAnimationStatus(): boolean;
    setNoise(noiseOrGenerator?: INoise | ((particle: Particle) => INoiseValue), init?: () => void, update?: () => void): void;
    destroy(): void;
    exportImg(callback: BlobCallback): void;
    exportImage(callback: BlobCallback, type?: string, quality?: number): void;
    exportConfiguration(): string;
    refresh(): Promise<void>;
    stop(): void;
    loadTheme(name?: string): Promise<void>;
    start(): Promise<void>;
    private init;
    private intersectionManager;
}
