import { Container } from "./Container";
import { IVelocity } from "./Interfaces/IVelocity";
import { IParticleValueAnimation } from "./Interfaces/IParticleValueAnimation";
import { ICoordinates } from "./Interfaces/ICoordinates";
import { IParticleImage } from "./Interfaces/IParticleImage";
import { Updater } from "./Particle/Updater";
import { IHsl, IRgb } from "./Interfaces/Colors";
import { IStroke } from "../Options/Interfaces/Particles/IStroke";
import { IShapeValues } from "../Options/Interfaces/Particles/Shape/IShapeValues";
import { IBubbleParticleData } from "./Interfaces/IBubbleParticleData";
import { IParticle } from "./Interfaces/IParticle";
import { IParticles } from "../Options/Interfaces/Particles/IParticles";
import { Particles } from "../Options/Classes/Particles/Particles";
import { MoveDirection, MoveDirectionAlt, ShapeType } from "../Enums";
import { RecursivePartial } from "../Types";
import { Infecter } from "./Particle/Infecter";
import { IDelta } from "./Interfaces/IDelta";
import { Mover } from "./Particle/Mover";
import { ILink } from "./Interfaces/ILink";
export declare class Particle implements IParticle {
    readonly id: number;
    private readonly container;
    pathAngle: number;
    destroyed: boolean;
    lifeDelay: number;
    lifeDelayTime: number;
    lifeDuration: number;
    lifeTime: number;
    livesRemaining: number;
    misplaced: boolean;
    spawning: boolean;
    lastNoiseTime: number;
    readonly noiseDelay: number;
    readonly updater: Updater;
    readonly infecter: Infecter;
    readonly mover: Mover;
    readonly sides: number;
    readonly strokeWidth: number;
    readonly options: Particles;
    links: ILink[];
    randomIndexData?: number;
    linksDistance?: number;
    linksWidth?: number;
    maxDistance?: number;
    moveSpeed?: number;
    sizeValue?: number;
    sizeAnimationSpeed?: number;
    readonly close: boolean;
    readonly direction: MoveDirection | keyof typeof MoveDirection | MoveDirectionAlt;
    readonly fill: boolean;
    readonly stroke: IStroke;
    readonly position: ICoordinates;
    readonly offset: ICoordinates;
    readonly shadowColor: IRgb | undefined;
    readonly color: IParticleValueAnimation<IHsl | undefined>;
    readonly opacity: IParticleValueAnimation<number>;
    readonly rotate: IParticleValueAnimation<number>;
    readonly size: IParticleValueAnimation<number>;
    readonly strokeColor: IParticleValueAnimation<IHsl | undefined>;
    readonly velocity: IVelocity;
    readonly shape: ShapeType | string;
    readonly image?: IParticleImage;
    readonly initialPosition: ICoordinates;
    readonly initialVelocity: IVelocity;
    readonly shapeData?: IShapeValues;
    readonly bubble: IBubbleParticleData;
    constructor(id: number, container: Container, position?: ICoordinates, overrideOptions?: RecursivePartial<IParticles>);
    move(delta: IDelta): void;
    update(delta: IDelta): void;
    draw(delta: IDelta): void;
    getPosition(): ICoordinates;
    getRadius(): number;
    getFillColor(): IHsl | undefined;
    getStrokeColor(): IHsl | undefined;
    destroy(): void;
    private calcPosition;
    private checkOverlap;
    private calculateVelocity;
    private loadImageShape;
}
